/*
  LCD - Library for most LCD charater displays, has common features 
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  p017 is an LCD chip that can be supplied with or without a PCB.
  p017 - serial varsion
  p018 - i2c version

*/  

#include "Arduino.h"
#include "p018.h"

// ==============================================================
// I2C PRIVATE
// ==============================================================
// ==============================================================
// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
int LCD::i2_16bit()
{
int rv;
  Wire.requestFrom(_i2adr, 2); // returns 2 bytes
  rv=Wire.read()*256; // high byte
  rv+=Wire.read(); // low byte
  return rv;
}

// =============================================================================
// I2C LCD display P018
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
LCD::LCD(char i2cadr)
{
char j;
  _i2adr=i2cadr;  
  Wire.begin(); // join i2c bus
}


// *****************************************************************************
// Clear screen on most displays is cmd 1
// *****************************************************************************
void LCD::lcd_cls()
{
    Wire.beginTransmission(_i2adr);
    Wire.write(2);
    Wire.write(1);
    Wire.endTransmission();
    delay(300);
}

// *****************************************************************************
// select a row and column
// row and column start at 0
// *****************************************************************************
void LCD::lcd_rowcol(char row, char col)
{
const int lcdrows[]={0x80,0xc0,0x94,0xd4}; 
int rc;
char buf[15];
    if(row < 0) row = 0;
    if(row > 3) row = 3;
    rc = lcdrows[row] + col;
    Wire.beginTransmission(_i2adr);
    Wire.write(2);
    Wire.write(rc);
    Wire.endTransmission();
}

// *****************************************************************************
// cursor control
// 0 turns cursot off
// 1 
// *****************************************************************************
void LCD::lcd_cursor(char cv)
{
char cmd;
    switch(cv) {
        case 0: { // turn off cursor
            cmd=12;
            break;
        }
        case 1: { // cursor on normal
            cmd=14;
            break;
        }
        case 2: { // blok flashing
            cmd=13;
            break;
        }
    }
    Wire.beginTransmission(_i2adr);
    Wire.write(2);
    Wire.write(cmd);
    Wire.endTransmission();
}

// *****************************************************************************
// Print overloaded 
// *****************************************************************************
// String
void LCD::lcd_print(char *value)
{
  Wire.beginTransmission(_i2adr);
  Wire.write(3); // data
  while(*value) Wire.write(*(value++));
  Wire.write(13); // always finish with 13  
  Wire.endTransmission();
  delay(1);
}
// Int
void LCD::lcd_print(int value)
{
char buf[15];
    sprintf(buf, "d%d", value);
    lcd_print(buf);
}

// *****************************************************************************
// back light control
// RGB has 3 back lights
// *****************************************************************************
void LCD::lcd_bl(char red, char green, char blue)
{
  Wire.beginTransmission(_i2adr);
  Wire.write(1); // back light
  Wire.write(red);
  Wire.write(green);
  Wire.write(blue);
  Wire.endTransmission();
}

// *****************************************************************************
// define custom character
// *****************************************************************************
void LCD::lcd_defcust(char position, char *definition)
{
char adr = 0x40;
char buf[15], *bp=definition;
    if(position > 7) position = 7;
    adr += 8 * position;
    // set address using command
    Wire.beginTransmission(_i2adr);
    Wire.write(2); // command
    Wire.write(adr);
    Wire.endTransmission();    
    delay(100);
    
    // now write definition
    Wire.beginTransmission(_i2adr);
    Wire.write(3); // data
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(*(bp++));
    Wire.write(13);
    Wire.endTransmission();
    delay(100);
}

// *****************************************************************************
// prints a custom char
// *****************************************************************************
void LCD::lcd_printcust(char address)
{
char buf[15];
    if(address > 7) address = 7;
    Wire.beginTransmission(_i2adr);
    Wire.write(3); // data
    Wire.write(address);
    Wire.write(13);
    Wire.endTransmission();
}    

